import os
import tempfile

from gtts import gTTS
from utils_base import Hash, Log

from utils_lang.Language import Language

log = Log('TTS')


class TTS:
    def __init__(self, lang: Language):
        self.lang = lang

    def get_cache_file_path(self, text: str) -> str:
        hash = Hash.md5(text)
        dir_temp = os.path.join(
            tempfile.gettempdir(), f'tts.{self.lang.iso2}'
        )
        os.makedirs(dir_temp, exist_ok=True)
        return os.path.join(dir_temp, f'{hash}.mp3')

    def write_nocache(self, text: str):
        cache_file_path = self.get_cache_file_path(text)
        tts = gTTS(text=text, lang=self.lang.iso2)
        tts.save(cache_file_path)
        log.debug(f'"{text}" -> "{cache_file_path}"')
        return cache_file_path

    def write(self, text: str):
        cache_file_path = self.get_cache_file_path(text)
        if not os.path.exists(cache_file_path):
            self.write_nocache(text)
        return cache_file_path



class Language:
    def __init__(self, name: str, iso2: str):
        self.name = name
        self.iso2 = iso2


Language.ENGLISH = Language('English', 'en')
Language.SINHALA = Language('Sinhala', 'si')
Language.TAMIL = Language('Tamil', 'ta')
Language.SPANISH = Language('Spanish', 'es')



import os
import tempfile
from functools import cache, cached_property

from deep_translator import GoogleTranslator
from utils_base import JSONFile, Log

from utils_lang.Language import Language

log = Log('Translator')


class Translator:
    def __init__(self, source_lang: Language, target_lang: Language):
        self.source_lang = source_lang
        self.target_lang = target_lang
        self.translator = GoogleTranslator(
            source=self.source_lang.iso2, target=target_lang.iso2
        )
        self.cache_idx = self.load_cache()

    @cached_property
    def cache_file_path(self) -> str:
        return os.path.join(
            tempfile.gettempdir(),
            f"Translate.{self.source_lang.iso2}-{self.target_lang.iso2}.json",
        )

    @property
    def cache_file(self) -> JSONFile:
        return JSONFile(self.cache_file_path)

    def load_cache(self) -> dict:
        return self.cache_file.read() if self.cache_file.exists else {}

    def save_cache(self):
        self.cache_file.write(self.cache_idx)

    def translate_nocache(self, text: str) -> str:
        translated_text = self.translator.translate(text)
        log.debug(f'"{text}" -> "{translated_text}"')
        return translated_text

    @cache
    def translate(self, text: str) -> str:
        if text in self.cache_idx:
            return self.cache_idx[text]

        translation = self.translate_nocache(text)
        self.cache_idx[text] = translation
        self.save_cache()

        return translation



import os
import shutil
import unittest

from utils_lang import TTS, Language


class TestTTS(unittest.TestCase):
    def test_write(self):
        for lang, text in [
            (
                Language.ENGLISH,
                "It is a truth universally acknowledged"
                + " that a single man in possession of a good fortune"
                + " must be in want of a wife.",
            ),
            (
                Language.SINHALA,
                "කොග්ගල වූ කලි එක් පැත්තකින් මුහුකින්ද අනික්"
                + " පැත්තෙන් සුන්දර වු නදියකින් ද සීමාවුනු බිම්තීරයකි.",
            ),
            (Language.TAMIL, "இதை எழுதியவர் நுவன்"),
        ]:
            tts = TTS(lang)

            tts.write_nocache(text)
            file_path = tts.write(text)
            self.assertTrue(os.path.exists(file_path))
            shutil.copyfile(
                file_path,
                os.path.join('tests', '_output', f'test_tts.{lang.iso2}.mp3'),
            )


if __name__ == '__main__':
    unittest.main()



import unittest

from utils_lang import Language, Translator


class TestTranslator(unittest.TestCase):
    def test_translate(self):
        for source_lang, target_lang, source_text, target_text in [
            (
                Language.ENGLISH,
                Language.SPANISH,
                "The sea is very clear.",
                'El mar es muy claro.',
            ),
            (
                Language.ENGLISH,
                Language.SINHALA,
                "Colombo is the capital of Sri Lanka.",
                'කොළඹ ශ්‍රී ලංකාවේ අගනුවරයි.',
            ),
            (
                Language.ENGLISH,
                Language.TAMIL,
                "what is the time?",
                'நேரம் என்ன?',
            ),
        ]:
            for source_lang, target_lang, source_text, target_text in [
                (source_lang, target_lang, source_text, target_text),
                (target_lang, source_lang, target_text, source_text),
            ]:
                translator = Translator(source_lang, target_lang)
                for translated_text in [
                    translator.translate(source_text),
                    translator.translate_nocache(source_text),
                ]:
                    self.assertEqual(
                        translated_text,
                        target_text,
                    )


if __name__ == '__main__':
    unittest.main()
